# Code to let some parameters not to vary by year (USING COXPH)
#
coxph.CRC <- function(DData,YData,XData,Margins=NULL, ...) {
    # DData = data matrix, consisting of c(0,1)
    # YData = Y design matrix
    # XData = X covariate matrix (include intercept)
    # Margins = a matrix for setting some parameteres in Lambda to zero. 
    #         
    require(survival)
    #
    D <- c(nrow(DData),ncol(DData),ncol(XData),nrow(YData),ncol(XData)*nrow(YData),NULL)
    RESP <- as.vector(t(DData))
    #
    Margins <- as.vector(Margins)
    if(is.null(Margins)) {Margins <- rep(1,D[5])}
    D[6] <- sum(Margins)
    VARS <- kronecker(XData,t(YData))
    VARS <- VARS[,Margins==1]
    # Fit the model as cox proportional hazards
    FIT <- coxph(Surv(2-RESP,RESP)~VARS+strata(rep(1:D[1],each=D[2])),method="breslow")
    Lambdas <- matrix(coef(FIT),D[3],D[4],T)
    LambdaVar <- matrix(diag(FIT$var),D[3],D[4],T)
    # Compute fitted values
    PopEst <- (1 + rowSums(exp(XData %*% Lambdas %*% YData)))/
        rowSums(exp(XData %*% Lambdas %*% YData))
    PopObs <- apply(DData,1,sum)
    PopMiss <- PopEst - PopObs
    Fitted <- exp(XData %*% Lambdas %*% YData) / rowSums(exp(XData %*% Lambdas %*% YData)) * 
        rowSums(DData)
    FittedProb <- Fitted / rowSums(Fitted)
    # Model parameters    
    Deviance <- - 2 * sum(DData * log(Fitted))# Deviance
    AIC <- - 2 * sum(DData * log(Fitted))+ 2 * length(coef(FIT))# AIC
    ## Variance computation
    temp.Y <- kronecker(matrix(rep(1,D[3]),nrow=1),FittedProb%*%t(YData))[,Margins==1]
    temp.0 <- matrix(YData %*% l.constraint ,nrow=D[1],ncol=D[5])[,Margins==1]
    temp.X <- kronecker(XData,matrix(rep(1,D[4]),nrow=1))[,Margins==1]
    #
    N.DERIV <- matrix(PopMiss,nrow=D[1],ncol=D[6])*temp.X*(temp.0-temp.Y)
    COND.VAR <- N.DERIV%*%FIT$var%*%t(N.DERIV)
    ASYM.VAR <- PopObs^2*PopMiss*(1+PopMiss)
#    
    return(list(FIT=FIT,Lambdas=Lambdas,LambdaVar=LambdaVar,
                PopTotal=sum(PopEst),Fitted=Fitted,PopEst=PopEst,
                Deviance=Deviance,AIC=AIC,CondVar=COND.VAR,AsymVar=ASYM.VAR))
}
