source("C:/data/statisticalmodelling/CoxPH.R")          # 'coxph' wrapper function for fitting Bock model
m <- 500;reps <- 1000                                   # total population
xxvar<-rnorm(m)                                         # standard normal variate
# probabilities of being listed by each of the three lists
    p1<-exp(0.5-0.5*xxvar)/(1+exp(0.5-0.5*xxvar))           
    p2<-exp(-0.5-0.5*xxvar)/(1+exp(-0.5-0.5*xxvar))
    p3<-exp(1+0.25*xxvar)/(1+exp(1+0.25*xxvar)) 
our.model<-NULL
big.model<-NULL
alho.model<-NULL
rho<-c(0,sqrt(c(0.1,0.2,0.3,0.4,0.5,1))) # normal correlation
cor.y<-matrix(0,nrow=reps,ncol=length(rho))
for(j in 1:reps){ #for(j in 1:1000){
    for(i in 1:length(rho)){
        # back transform from probability to quantile
        th1<-qnorm(p1)
        th2<-qnorm(p2)
        th3<-qnorm(p3)
        gam<-rho[i]
            eps0<-rnorm(m)
            eps1<-rnorm(m)
            eps2<-rnorm(m)
            eps3<-rnorm(m)
            v1<-runif(m)
            v2<-runif(m)
            u1<-ifelse(v1<=gam,1,0)
            u2<-ifelse(v2<=gam,1,0)
            z1<-u1*eps0+(1-u1)*eps1
            z2<-u2*eps0+(1-u2)*eps2
            z3<-eps3
            y1<-ifelse(z1<=th1,1,0)
            y2<-ifelse(z2<=th2,1,0)
            y3<-ifelse(z3<=th3,1,0)
            biny<-cbind(y1,y2,y3)
            cor.y[j,i]<-cor(biny)[1,2]
        # creating data file: remove individual not listed in all lists
            y1<-na.exclude(ifelse(biny[,1]==0 & biny[,2]==0 & biny[,3]==0,NA,y1))
            y2<-na.exclude(ifelse(biny[,1]==0 & biny[,2]==0 & biny[,3]==0,NA,y2))
            y3<-na.exclude(ifelse(biny[,1]==0 & biny[,2]==0 & biny[,3]==0,NA,y3))
        xvar<-na.exclude(ifelse(biny[,1]==0 & biny[,2]==0 & biny[,3]==0,NA,xxvar))
            mi<-ifelse(y1==1&y2==1&y3==1,1,0)
            u1i<-ifelse(y1==1&y2==0&y3==0,1,0)
            u2i<-ifelse(y1==0&y2==1&y3==0,1,0)
            u3i<-ifelse(y1==0&y2==0&y3==1,1,0)
            u12i<-ifelse(y1==1&y2==1&y3==0,1,0)
            u13i<-ifelse(y1==1&y2==0&y3==1,1,0)
            u23i<-ifelse(y1==0&y2==1&y3==1,1,0)
        tdata<-matrix(cbind(u1i,u2i,u3i,u12i,u13i,u23i,mi),ncol=7) # data file
        xdata<-cbind(rep(1,nrow(tdata)),xvar)                      # covariate matrix
        lcon<-c(1,1,1,-1,-1,-1,1)
        # analysis
        ydata<-matrix(c(
            1,0,0,1,1,0,1,
            0,1,0,1,0,1,1,
            0,0,1,0,1,1,1,
            0,0,0,1,0,0,1),nrow=4,ncol=7,T)
        test<-coxph.CRC(tdata, ydata, xdata,lcon)
        our.model<-rbind(our.model, cbind(test$AIC,test$PopTotal,sum(test$CondVar)+sum(test$AsymVar)))
        #
        ydata<-matrix(c(
            1,0,0,1,1,0,1,
            0,1,0,1,0,1,1,
            0,0,1,0,1,1,1,
            0,0,0,1,0,0,1,
            0,0,0,0,1,0,1),nrow=5,ncol=7,T)
        test<-coxph.CRC(tdata, ydata, xdata,lcon)
        big.model<-rbind(big.model, cbind(test$AIC,test$PopTotal,sum(test$CondVar)+sum(test$AsymVar)))
        #
        ydata<-matrix(c(
            1,0,0,1,1,0,1,
            0,1,0,1,0,1,1,
            0,0,1,0,1,1,1),nrow=3,ncol=7,T)
        test<-coxph.CRC(tdata, ydata, xdata,lcon)
        alho.model<-rbind(alho.model, cbind(test$AIC,test$PopTotal,sum(test$CondVar)+sum(test$AsymVar)))
    }
    if(!(j %% 20)) {cat(".")}
}
#################################
# Computations for tables
test<-cbind(our.model,alho.model)
strata.alho<-list(rep(c(0,0.1,0.2,0.3,0.4,0.5,1),reps))
our.cov<-ifelse(our.model[,2]-1.96*sqrt(our.model[,3])<=500 & our.model[,2]+1.96*sqrt(our.model[,3])>=500,1,0)
big.cov<-ifelse(big.model[,2]-1.96*sqrt(big.model[,3])<=500 & big.model[,2]+1.96*sqrt(big.model[,3])>=500,1,0)
alho.cov<-ifelse(alho.model[,2]-1.96*sqrt(alho.model[,3])<=500 & alho.model[,2]+1.96*sqrt(alho.model[,3])>=500,1,0)
#################################
# The tables
cbind(round(apply(cor.y,2,mean),2),
round(tapply(alho.model[,1],strata.alho,median)),
round(tapply(alho.model[,2],strata.alho,mean),1),
round(tapply(alho.model[,2],strata.alho,min),1),
round(tapply(alho.model[,2],strata.alho,max),1),
round(tapply(sqrt(alho.model[,3]),strata.alho,mean),2),
tapply(alho.cov,strata.alho,sum)/10)

cbind(round(apply(cor.y,2,mean),2),
round(tapply(our.model[,1],strata.alho,median)),
round(tapply(our.model[,2],strata.alho,mean),1),
round(tapply(our.model[,2],strata.alho,min),1),
round(tapply(our.model[,2],strata.alho,max),1),
round(tapply(sqrt(our.model[,3]),strata.alho,mean),2),
tapply(our.cov,strata.alho,sum)/10)

cbind(round(apply(cor.y,2,mean),2),
round(tapply(big.model[,1],strata.alho,median)),
round(tapply(big.model[,2],strata.alho,mean),1),
round(tapply(big.model[,2],strata.alho,min),1),
round(tapply(big.model[,2],strata.alho,max),1),
round(tapply(sqrt(big.model[,3]),strata.alho,mean),2),
tapply(big.cov,strata.alho,sum)/10)
